@@ -29,10 +29,12 @@
 #include <QtWidgets/QStyledItemDelegate>
 #include <QtWebEngineWidgets/QWebEngineSettings>
 #include <QFontDialog>
+#include <QFileInfo>
 
 #include "AppearanceWidget.h"
 #include "Misc/SettingsStore.h"
 #include "Misc/Utility.h"
+#include "sigil_constants.h"
 
 class ColorSwatchDelegate : public QStyledItemDelegate
 {
@@ -82,10 +84,28 @@ AppearanceWidget::AppearanceWidget()
 
 #ifdef Q_OS_MAC
     // Disable the HighDPI combobox on Mac
+    // Effectively an isMacOS runtime check
     m_isHighDPIComboEnabled = false;
 #endif
 
     ui.setupUi(this);
+    ui.Default->setEnabled(true);
+    ui.Fluent->setEnabled(true);
+    ui.Material->setEnabled(true);
+    ui.Custom->setEnabled(QFileInfo(Utility::DefinePrefsDir() + "/" + CUSTOM_ICON_THEME_FILENAME).exists());
+    
+    ui.Custom->setToolTip(tr("Custom icon theme provided by the user"));
+    // attempt to use a png image as a tooltip for the icon theme selection
+    if (Utility::IsDarkMode()) {
+        ui.Default->setToolTip("<img src=':/icon/Main_dark.png'>");
+        ui.Fluent->setToolTip("<img src=':/icon/Fluent_dark.png'>");
+        ui.Material->setToolTip("<img src=':/icon/Material_dark.png'>");
+    } else {
+        ui.Default->setToolTip("<img src=':/icon/Main.png'>");
+        ui.Fluent->setToolTip("<img src=':/icon/Fluent.png'>");
+        ui.Material->setToolTip("<img src=':/icon/Material.png'>");
+    }
+    
     // Custom delegate for painting the color swatches
     ui.codeViewColorsList->setItemDelegate(new ColorSwatchDelegate(ui.codeViewColorsList));
     ui.comboHighDPI->addItems({tr("Detect"), tr("On"), tr("Off")});
@@ -98,6 +118,13 @@ AppearanceWidget::AppearanceWidget()
     ui.comboHighDPI->setToolTip(highdpi_tooltip);
     // The HighDPI setting is unused/unnecessary on Mac
     ui.comboHighDPI->setEnabled(m_isHighDPIComboEnabled);
+    QString drag_tweak_tooltip = "<p>" + tr("Adjust the distance necessary to drag an item before a move event is triggered.");
+    drag_tweak_tooltip += "<p>" + tr("-20 to +20 pixel range");
+    ui.dragTweakSpinBox->setToolTip(drag_tweak_tooltip);
+    ui.dragTweakSpinBox->setMinimum(-20);
+    ui.dragTweakSpinBox->setMaximum(20);
+    // The Drag start-distance setting is unused/unnecessary on Mac
+    ui.dragTweakSpinBox->setEnabled(m_isHighDPIComboEnabled);
     m_codeViewAppearance = readSettings();
     loadCodeViewColorsList(m_codeViewAppearance);
     m_uiFontResetFlag = false;
@@ -110,9 +137,20 @@ PreferencesWidget::ResultActions AppearanceWidget::saveSettings()
     settings.setAppearancePrefsTabIndex(ui.tabAppearance->currentIndex());
     settings.setShowFullPathOn(ui.ShowFullPath->isChecked() ? 1 : 0);
     settings.setPreviewDark(ui.PreviewDarkInDM->isChecked() ? 1 : 0);
+    // handle icon theme
+    QString icon_theme = "main";
+    if (ui.Fluent->isChecked()) {
+        icon_theme = "fluent";
+    } else if (ui.Material->isChecked()) {
+        icon_theme = "material";
+    } else if (ui.Custom->isEnabled() && ui.Custom->isChecked()) {
+        icon_theme = "custom";
+    }	
+    settings.setUIIconTheme(icon_theme);
     // Don't try to get the index of a disabled combobox
     if (m_isHighDPIComboEnabled) {
         settings.setHighDPI(ui.comboHighDPI->currentIndex());
+        settings.setUiDragDistanceTweak(ui.dragTweakSpinBox->value());
     }
     settings.setUIFont(m_currentUIFont);
     SettingsStore::PreviewAppearance PVAppearance;
@@ -195,11 +233,18 @@ PreferencesWidget::ResultActions AppearanceWidget::saveSettings()
     if (m_PreviewDark != (ui.PreviewDarkInDM->isChecked() ? 1 : 0)) {
         results = results | PreferencesWidget::ResultAction_ReloadPreview;
     }
+    // if icon theme change set need for restart
+    if (m_currentIconTheme != icon_theme) {
+        results = results | PreferencesWidget::ResultAction_RestartSigil;
+    }
     // Don't try to get the index of a disabled combobox
     if (m_isHighDPIComboEnabled) {
         if (m_HighDPI != (ui.comboHighDPI->currentIndex())) {
             results = results | PreferencesWidget::ResultAction_RestartSigil;
         }
+        if (m_DragTweak != (ui.dragTweakSpinBox->value())) {
+            results = results | PreferencesWidget::ResultAction_RestartSigil;
+        }
     }
     if ((m_currentUIFont != m_initUIFont) || m_uiFontResetFlag) {
         results = results | PreferencesWidget::ResultAction_RestartSigil;
@@ -215,10 +260,27 @@ SettingsStore::CodeViewAppearance AppearanceWidget::readSettings()
     ui.tabAppearance->setCurrentIndex(settings.appearancePrefsTabIndex());
     m_ShowFullPathOn = settings.showFullPathOn();
     ui.ShowFullPath->setChecked(settings.showFullPathOn());
-    // Don't try to set the index of a disabled combobox
+
+    // Handle Icon Theme
+    QString icon_theme = settings.uiIconTheme();
+    m_currentIconTheme = icon_theme;
+    if (ui.Custom->isEnabled()) {
+	ui.Custom->setChecked(icon_theme == "custom");
+    } else {
+	ui.Custom->setChecked(false);
+	if (icon_theme == "custom") {
+	    icon_theme = "main";
+	}
+    }
+    ui.Default->setChecked(icon_theme == "main");
+    ui.Fluent->setChecked(icon_theme == "fluent");
+    ui.Material->setChecked(icon_theme == "material");
+    // Don't try to set the index of disabled widgets
     if (m_isHighDPIComboEnabled) {
         m_HighDPI = settings.highDPI();
         ui.comboHighDPI->setCurrentIndex(m_HighDPI);
+        m_DragTweak = settings.uiDragDistanceTweak();
+        ui.dragTweakSpinBox->setValue(m_DragTweak);
     }
     if (!settings.uiFont().isEmpty()) {
         m_initUIFont = settings.uiFont();
