@@ -1,6 +1,6 @@
 /************************************************************************
 **
-**  Copyright (C) 2015-2019 Kevin Hendricks, Statford, ON 
+**  Copyright (C) 2015-2020 Kevin Hendricks, Statford, ON 
 **  Copyright (C) 2012      Dave Heiland
 **  Copyright (C) 2012      John Schember <john@nachtimwald.com>
 **
@@ -346,7 +346,7 @@ void ImageFilesWidget::Save()
 
     try {
         Utility::WriteUnicodeTextFile(report_info, destination);
-    } catch (CannotOpenFile) {
+    } catch (CannotOpenFile&) {
         QMessageBox::warning(this, tr("Sigil"), tr("Cannot save report file."));
     }
 
@@ -432,7 +432,9 @@ void ImageFilesWidget::OpenContextMenu(const QPoint &point)
 {
     SetupContextMenu(point);
     m_ContextMenu->exec(ui.fileTree->viewport()->mapToGlobal(point));
-    m_ContextMenu->clear();
+    if (!m_ContextMenu.isNull()) {
+        m_ContextMenu->clear();
+    }
 }
 
 void ImageFilesWidget::SetupContextMenu(const QPoint &point)
