@@ -362,7 +362,7 @@ void LinksWidget::Save()
 
     try {
         Utility::WriteUnicodeTextFile(report_info, destination);
-    } catch (CannotOpenFile) {
+    } catch (CannotOpenFile&) {
         QMessageBox::warning(this, tr("Sigil"), tr("Cannot save report file."));
     }
 
