@@ -1,8 +1,8 @@
 /************************************************************************
 **
-**  Copyright (C) 2016 - 2019 Kevin B. Hendricks, Stratford, Ontario, Canada
-**  Copyright (C) 2012 John Schember <john@nachtimwald.com>
-**  Copyright (C) 2009, 2010, 2011  Strahinja Markovic  <strahinja.markovic@gmail.com>
+**  Copyright (C) 2016-2020 Kevin B. Hendricks, Stratford, Ontario, Canada
+**  Copyright (C) 2012      John Schember <john@nachtimwald.com>
+**  Copyright (C) 2009-2011 Strahinja Markovic  <strahinja.markovic@gmail.com>
 **
 **  This file is part of Sigil.
 **
@@ -51,6 +51,7 @@
 #include "BookManipulation/FolderKeeper.h"
 #include "BookManipulation/CleanSource.h"
 #include "Importers/ImportEPUB.h"
+#include "Misc/MediaTypes.h"
 #include "Misc/FontObfuscation.h"
 #include "Misc/HTMLEncodingResolver.h"
 #include "Misc/QCodePage437Codec.h"
@@ -62,7 +63,6 @@
 #include "ResourceObjects/NCXResource.h"
 #include "ResourceObjects/Resource.h"
 #include "ResourceObjects/OPFParser.h"
-#include "SourceUpdates/UniversalUpdates.h"
 #include "sigil_constants.h"
 #include "sigil_exception.h"
 
@@ -104,7 +104,7 @@ ImportEPUB::ImportEPUB(const QString &fullfilepath)
 // and returns the created Book
 QSharedPointer<Book> ImportEPUB::GetBook(bool extract_metadata)
 {
-    QList<XMLResource *> non_well_formed;
+    QList<HTMLResource *> non_well_formed;
     SettingsStore ss;
 
     if (!Utility::IsFileReadable(m_FullFilePath)) {
@@ -121,25 +121,19 @@ QSharedPointer<Book> ImportEPUB::GetBook(bool extract_metadata)
 
     QApplication::setOverrideCursor(Qt::WaitCursor);
 
-    // These mutate the m_Book object
     LocateOPF();
     m_opfDir = QFileInfo(m_OPFFilePath).dir();
     // These mutate the m_Book object
     ReadOPF();
     AddObfuscatedButUndeclaredFonts(encrypted_files);
     AddNonStandardAppleXML();
+
+    m_Book->GetFolderKeeper()->SetGroupFolders(m_ManifestFilePaths, m_ManifestMediaTypes);
+
     LoadInfrastructureFiles();
-    qDebug() << "m_ExtractedFolderPath: " << m_ExtractedFolderPath;
+
     // Check for files missing in the Manifest and create warning
     QStringList notInManifest;
-    qDebug() << "Zip File Paths: ";
-    foreach(QString fp, m_ZipFilePaths) {
-        qDebug() << "    " <<  fp;
-    }
-    qDebug() << "Manifest File Paths: ";
-    foreach(QString fp, m_ManifestFilePaths) {
-        qDebug() << "    " << fp;
-    }
     foreach(QString file_path, m_ZipFilePaths) {
         // skip mimetype and anything in META-INF and the opf itself
         if (file_path == "mimetype") continue;
@@ -149,13 +143,14 @@ QSharedPointer<Book> ImportEPUB::GetBook(bool extract_metadata)
 	    notInManifest << file_path;
 	}
     }
-    qDebug() << "missing file paths: " << notInManifest;
     
     if (!notInManifest.isEmpty()) {
         Utility::DisplayStdWarningDialog(tr("Files exist in epub that are not listed in the manifest, they will be ignored"), notInManifest.join("\n"));
     }
-    const QHash<QString, QString> updates = LoadFolderStructure();
-    const QList<Resource *> resources     = m_Book->GetFolderKeeper()->GetResourceList();
+
+    LoadFolderStructure();
+
+    const QList<Resource *> resources = m_Book->GetFolderKeeper()->GetResourceList();
 
     // We're going to check all html files until we find one that isn't well formed then we'll prompt
     // the user if they want to auto fix or not.
@@ -179,9 +174,21 @@ QSharedPointer<Book> ImportEPUB::GetBook(bool extract_metadata)
                 }
             }
             if (ss.cleanOn() & CLEANON_OPEN) {
-              if (!XhtmlDoc::IsDataWellFormed(hresource->GetText(),hresource->GetEpubVersion())) {
+                if (!XhtmlDoc::IsDataWellFormed(hresource->GetText(),hresource->GetEpubVersion())) {
                     non_well_formed << hresource;
-                }
+                } else {
+		    QString txt = hresource->GetText();
+		    // had cases of large files with no line breaks
+		    if (txt.size() > 307200) {
+			int lines = 0;
+			QChar *uc = txt.data();
+			QChar *e = uc + txt.size();
+			for (; uc != e; ++uc) {
+			    if (uc->unicode() == 0x000A) lines++;
+			}
+			if (lines < 5) non_well_formed << hresource;
+		    }
+		}
             }
         }
     }
@@ -189,24 +196,23 @@ QSharedPointer<Book> ImportEPUB::GetBook(bool extract_metadata)
         QApplication::restoreOverrideCursor();
         if (QMessageBox::Yes == QMessageBox::warning(QApplication::activeWindow(),
                 tr("Sigil"),
-                tr("This EPUB has HTML files that are not well formed. "
-                   "Sigil can attempt to automatically fix these files, although this "
-                   "can result in data loss.\n\n"
+                tr("This EPUB has HTML files that are not well formed or are "
+                   "missing a DOCTYPE, html, head or body elements. "
+                   "Sigil can automatically fix these files, although "
+                   "this may result in minor data loss in extreme circumstances.\n\n"
                    "Do you want to automatically fix the files?"),
-                QMessageBox::Yes|QMessageBox::No)
-           ) {
+                QMessageBox::Yes|QMessageBox::No)) 
+        {
+	    foreach(HTMLResource* htmlres, non_well_formed) {
+		QString fixed_text = CleanSource::Mend(htmlres->GetText(),htmlres->GetEpubVersion());
+                htmlres->SetText(fixed_text);
+	    }
             non_well_formed.clear();
         }
         QApplication::setOverrideCursor(Qt::WaitCursor);
     }
 
-    const QStringList load_errors = UniversalUpdates::PerformUniversalUpdates(false, resources, updates, non_well_formed);
-
-    Q_FOREACH (QString err, load_errors) {
-        AddLoadWarning(QString("%1").arg(err));
-    }
-
-    ProcessFontFiles(resources, updates, encrypted_files);
+    ProcessFontFiles(resources, encrypted_files);
 
     if (m_PackageVersion.startsWith('3')) {
         HTMLResource * nav_resource = NULL;
@@ -253,6 +259,13 @@ QSharedPointer<Book> ImportEPUB::GetBook(bool extract_metadata)
                        QObject::tr("Sigil has created a new one for you."));
     }
 
+    // update the ShortPathNames to reflect any name duplication
+    m_Book->GetFolderKeeper()->updateShortPathNames();
+
+    // since we no longer run universal updates we should run 
+    // InitialLoad on all TextResources to make sure everything gets loaded
+    m_Book->GetFolderKeeper()->PerformInitialLoads();
+
     // If we have modified the book to add spine attribute, manifest item or NCX mark as changed.
     m_Book->SetModified(GetLoadWarnings().count() > 0);
     QApplication::restoreOverrideCursor();
@@ -280,7 +293,11 @@ QHash<QString, QString> ImportEPUB::ParseEncryptionXml()
             if (encryption.name() == "EncryptionMethod") {
                 encryption_algo = encryption.attributes().value("", "Algorithm").toString();
             } else if (encryption.name() == "CipherReference") {
+                // Note: fragments are not part of the CipherReference specs so this is okay
                 uri = Utility::URLDecodePath(encryption.attributes().value("", "URI").toString());
+                // hack to handle non-spec encryption file url relative to META-INF instead
+                // of being absolute from epub root as the spec calls for
+                if (uri.startsWith("../")) uri = uri.mid(3,-1);
                 encrypted_files[ uri ] = encryption_algo;
             }
         }
@@ -353,11 +370,8 @@ void ImportEPUB::AddNonStandardAppleXML()
 
 
 // Each resource can provide us with its new path. encrypted_files provides
-// a mapping from old resource paths to the obfuscation algorithms.
-// So we use the updates hash which provides a mapping from old paths to new
-// paths to match the resources to their algorithms.
+// a mapping from the resource paths to the obfuscation algorithms.
 void ImportEPUB::ProcessFontFiles(const QList<Resource *> &resources,
-                                  const QHash<QString, QString> &updates,
                                   const QHash<QString, QString> &encrypted_files)
 {
     if (encrypted_files.empty()) {
@@ -370,22 +384,9 @@ void ImportEPUB::ProcessFontFiles(const QList<Resource *> &resources,
         return;
     }
 
-    QHash<QString, QString> new_font_paths_to_algorithms;
-    foreach(QString old_update_path, updates.keys()) {
-        if (!FONT_EXTENSIONS.contains(QFileInfo(old_update_path).suffix().toLower())) {
-            continue;
-        }
-
-        QString new_update_path = updates[ old_update_path ];
-        foreach(QString old_encrypted_path, encrypted_files.keys()) {
-            if (old_update_path == old_encrypted_path) {
-                new_font_paths_to_algorithms[ new_update_path ] = encrypted_files[ old_encrypted_path ];
-            }
-        }
-    }
     foreach(FontResource * font_resource, font_resources) {
-        QString match_path = "../" + font_resource->GetRelativePathToOEBPS();
-        QString algorithm  = new_font_paths_to_algorithms.value(match_path);
+        QString match_path = font_resource->GetRelativePath();
+        QString algorithm  = encrypted_files.value(match_path);
 
         if (algorithm.isEmpty()) {
             continue;
@@ -490,7 +491,7 @@ void ImportEPUB::ExtractContainer()
                     dir.mkpath(qfile_name);
                     continue;
                 } else {
-                    dir.mkpath(qfile_info.path());
+		    if (!qfile_info.path().isEmpty()) dir.mkpath(qfile_info.path());
 		    // add it to the list of files found inside the zip
 		    if (cp437_file_name.isEmpty()) {
 		        m_ZipFilePaths << qfile_name;
@@ -560,7 +561,7 @@ void ImportEPUB::LocateOPF()
     QXmlStreamReader container;
     try {
         container.addData(Utility::ReadUnicodeTextFile(fullpath));
-    } catch (CannotOpenFile) {
+    } catch (CannotOpenFile&) {
         // Find the first OPF file.
         QString OPFfile;
         QDirIterator files(m_ExtractedFolderPath, QStringList() << "*.opf", QDir::NoFilter, QDirIterator::Subdirectories);
@@ -672,6 +673,12 @@ void ImportEPUB::ReadOPF()
         throw (EPUBLoadParseError(error.toStdString()));
     }
 
+    
+    //Important!  The OPF Resource in the new book must be created now before adding to it in any way
+    QString bookpath;
+    bookpath = m_OPFFilePath.right(m_OPFFilePath.length() - m_ExtractedFolderPath.length() - 1);
+    m_Book->GetFolderKeeper()->AddOPFToFolder(m_PackageVersion, bookpath);
+
     // Ensure we have an NCX available
     LocateOrCreateNCX(ncx_id_on_spine);
 
@@ -738,49 +745,71 @@ void ImportEPUB::ReadManifestItemElement(QXmlStreamReader *opf_reader)
     QString href = opf_reader->attributes().value("", "href").toString();
     QString type = opf_reader->attributes().value("", "media-type").toString();
     QString properties = opf_reader->attributes().value("", "properties").toString();
-    // Paths are percent encoded in the OPF, we use "normal" paths internally.
-    href = Utility::URLDecodePath(href);
-    QString extension = QFileInfo(href).suffix().toLower();
+    // FIXME: can epub3 OPF Manifest href attributes include fragments?
+    // FIXME: under epub2 fragments are explicitly outlawed in spec
+    // For robustness sake we will assume they can but ...
+    // Note:  Under epub3 they can point outside the epub so need to handle full url
 
-    // find the epub root relative file path from the opf location and the item href
-    QString file_path = m_opfDir.absolutePath() + "/" + href;
-    qDebug() << "creating manifest file path from: " << file_path;
-    file_path = Utility::resolveRelativeSegmentsInFilePath(file_path,"/");
-    file_path = file_path.remove(0, m_ExtractedFolderPath.length() + 1); 
+    QString apath;
+    if (href.indexOf(':') == -1) {
+	// we know we have a relative href to a file so no fragments can exist
+	apath = Utility::URLDecodePath(href);
+    }
+    // for hrefs pointing outside the epub, apath will be empty
+    // qDebug() << "ImportEpub with Manifest item: " << href << apath;
+    QString extension = QFileInfo(apath).suffix().toLower();
+
+    // validate the media type if we can, and warn otherwise
+    QString group = MediaTypes::instance()->GetGroupFromMediaType(type,"");
+    QString ext_mtype = MediaTypes::instance()->GetMediaTypeFromExtension(extension, "");
+    if (type.isEmpty() || group.isEmpty()) {
+	const QString load_warning = QObject::tr("The OPF uses an unrecognized media type \"%1\" for file \"%2\"").arg(type).arg(QFileInfo(apath).fileName()) +
+	    " - " + QObject::tr("A temporary media type of \"%1\" has been assigned. You should edit your OPF file to fix this problem.").arg(ext_mtype);
+        AddLoadWarning(load_warning);
+    }
+
+    if (!apath.isEmpty()) {
+        // find the epub root relative file path from the opf location and the item href
+        QString file_path = m_opfDir.absolutePath() + "/" + apath;
+        file_path = Utility::resolveRelativeSegmentsInFilePath(file_path,"/");
+        file_path = file_path.remove(0, m_ExtractedFolderPath.length() + 1); 
     
-    if (type != NCX_MIMETYPE && extension != NCX_EXTENSION) {
-        if (!m_ManifestFilePaths.contains(file_path)) {
-            if (m_Files.contains(id)) {
-                // We have an error situation with a duplicate id in the epub.
-                // We must warn the user, but attempt to use another id so the epub can still be loaded.
-                QString base_id = QFileInfo(href).fileName();
-                QString new_id(base_id);
-                int duplicate_index = 0;
+        if (type != NCX_MIMETYPE && extension != NCX_EXTENSION) {
+            if (!m_ManifestFilePaths.contains(file_path)) {
+                if (m_Files.contains(id)) {
+                    // We have an error situation with a duplicate id in the epub.
+                    // We must warn the user, but attempt to use another id so the epub can still be loaded.
+                    QString base_id = QFileInfo(apath).fileName();
+                    QString new_id(base_id);
+                    int duplicate_index = 0;
 
-                while (m_Files.contains(new_id)) {
-                    duplicate_index++;
-                    new_id = QString("%1%2").arg(base_id).arg(duplicate_index);
+                    while (m_Files.contains(new_id)) {
+                        duplicate_index++;
+                        new_id = QString("%1%2").arg(base_id).arg(duplicate_index);
+                    }
+
+                    const QString load_warning = QObject::tr("The OPF manifest contains duplicate ids for: %1").arg(id) +
+                  " - " + QObject::tr("A temporary id has been assigned to load this EPUB. You should edit your OPF file to remove the duplication.");
+                    id = new_id;
+                    AddLoadWarning(load_warning);
                 }
 
-                const QString load_warning = QObject::tr("The OPF manifest contains duplicate ids for: %1").arg(id) +
-                  " - " + QObject::tr("A temporary id has been assigned to load this EPUB. You should edit your OPF file to remove the duplication.");
-                id = new_id;
-                AddLoadWarning(load_warning);
-            }
+                m_Files[ id ] = apath;
+                m_FileMimetypes[ id ] = type;
+                m_ManifestFilePaths << file_path;
+	        m_ManifestMediaTypes << type;
 
-            m_Files[ id ] = href;
-            m_FileMimetypes[ id ] = type;
-            m_ManifestFilePaths << file_path;
-
-            // store information about any nav document
-            if (properties.contains("nav")) {
-                m_NavId = id;
-                m_NavHref = href;
+                // store information about any nav document
+                if (properties.contains("nav")) {
+                    m_NavId = id;
+                    m_NavHref = apath;
+                }
             }
+        } else {
+            m_NcxCandidates[ id ] = apath;
+	    m_ManifestFilePaths << file_path;
+	    m_ManifestMediaTypes << type;
         }
-    } else {
-        m_NcxCandidates[ id ] = href;
-	m_ManifestFilePaths << file_path;
     }
 }
 
@@ -791,82 +820,91 @@ void ImportEPUB::LocateOrCreateNCX(const QString &ncx_id_on_spine)
     QString ncx_href = "";
     m_NCXId = ncx_id_on_spine;
 
-    // if epub2 must have an ncx
-    if (m_PackageVersion.startsWith('2')) {
-        m_Book->GetFolderKeeper()->AddNCXToFolder(m_PackageVersion);
+    // handle the normal/proper case of an ncx id on the spine matching an ncx candidate that exists
+    if (!m_NCXId.isEmpty() && m_NcxCandidates.contains(m_NCXId)) {
+        QString bookpath;
+        ncx_href = m_NcxCandidates[ m_NCXId ];
+        m_NCXFilePath = QFileInfo(m_OPFFilePath).absolutePath() % "/" % ncx_href;
+	m_NCXFilePath = Utility::resolveRelativeSegmentsInFilePath(m_NCXFilePath, "/");
+	bookpath = m_NCXFilePath.right(m_NCXFilePath.length() - m_ExtractedFolderPath.length() - 1);
+        m_Book->GetFolderKeeper()->AddNCXToFolder(m_PackageVersion, bookpath);
+        m_NCXNotInManifest = false;
+        return;
     }
 
-    // Handle various failure conditions, such as:
-    // - ncx not specified in the spine (search for a matching manifest item by extension)
-    // - ncx specified in spine, but no matching manifest item entry (create a new one)
-    // - ncx file not physically present (create a new one)
-    // - ncx not in spine or manifest item (create a new one)
-    if (!m_NCXId.isEmpty()) {
-        ncx_href = m_NcxCandidates[ m_NCXId ];
-    } else {
-        // Search for the ncx in the manifest by looking for files with
-        // a .ncx extension.
-        QHashIterator<QString, QString> ncxSearch(m_NcxCandidates);
+    bool found = false;
 
+    // now handle ncx not specified in spine but file with ncx extension exists in manifest
+    // Search for the ncx in the manifest by looking for files with
+   // a .ncx extension.
+    if (m_NCXId.isEmpty()) {
+
+        QHashIterator<QString, QString> ncxSearch(m_NcxCandidates);
         while (ncxSearch.hasNext()) {
             ncxSearch.next();
 
             if (QFileInfo(ncxSearch.value()).suffix().toLower() == NCX_EXTENSION) {
+                // we found a file with an ncx extension
                 m_NCXId = ncxSearch.key();
-                load_warning = QObject::tr("The OPF file did not identify the NCX file correctly.") + "\n" + 
+                found = true;
+		break;
+            }
+        }
+    }
+
+    if (found) {
+        // m_NCXId has been properly set
+        ncx_href = m_NcxCandidates[ m_NCXId ];
+        m_NCXFilePath = QFileInfo(m_OPFFilePath).absolutePath() % "/" % ncx_href;
+	m_NCXFilePath = Utility::resolveRelativeSegmentsInFilePath(m_NCXFilePath, "/");
+
+	QString bookpath = m_NCXFilePath.right(m_NCXFilePath.length() - m_ExtractedFolderPath.length() - 1);
+        m_Book->GetFolderKeeper()->AddNCXToFolder(m_PackageVersion, bookpath);
+        m_NCXNotInManifest = false;
+        load_warning = QObject::tr("The OPF file did not identify the NCX file correctly.") + "\n" + 
                                " - "  +  QObject::tr("Sigil has used the following file as the NCX:") + 
                                QString(" %1").arg(m_NcxCandidates[ m_NCXId ]);
-                ncx_href = m_NcxCandidates[ m_NCXId ];
-                break;
-            }
-        }
-    }
-    if (!ncx_href.isEmpty()) {
-        m_NCXFilePath = QFileInfo(m_OPFFilePath).absolutePath() % "/" % ncx_href;
 
-	if (QFile::exists(m_NCXFilePath) && m_PackageVersion.startsWith('3')) {
-	    // only ask folderkeeper to create an NCX if an ncx already exists in this epub3
-            m_Book->GetFolderKeeper()->AddNCXToFolder(m_PackageVersion);
-	}
+        AddLoadWarning(load_warning);
+        return;
     }
 
-    if (ncx_href.isEmpty() || m_NCXFilePath.isEmpty() || !QFile::exists(m_NCXFilePath)) {
-        m_NCXNotInManifest = m_NCXId.isEmpty() || ncx_href.isEmpty();
-        m_NCXId.clear();
+    // An NCX is only required in epub2 so punt here if epub3
+    if ( m_PackageVersion.startsWith('3') ) return;
 
-        // For epub3 having an ncx is optional so do nothing here
+    // epub2 only here
 
-        // For epub2, having an ncx is required so things are really
-        // bad and no .ncx file was found in the manifest or
-        // the file does not physically exist.  We need to create a new one.
+    // If we reached here there is no file with an ncx file extension in the manifest
+    // There might be a file with an ncx extension inside the epub zip folder but 
+    // since it was unmanifested, we will not use it anyway.
+    // So we need to create a new one and thereby handle the following 
+    // failure conditions:
+    //     - ncx specified in spine, but no matching manifest item entry
+    //     - ncx file not physically present
+    //     - ncx not in spine or manifest item
 
-        if ( m_PackageVersion.startsWith('2') ) {
-            m_NCXFilePath = QFileInfo(m_OPFFilePath).absolutePath() % "/" % NCX_FILE_NAME;
+    m_NCXNotInManifest = true;
 
-            // Create a new file for the NCX.
-            NCXResource ncx_resource(m_ExtractedFolderPath, m_NCXFilePath, NULL);
-            ncx_resource.SetEpubVersion(m_PackageVersion);
-
-            // We are relying on an identifier being set from the metadata.
-            // It might not have one if the book does not have the urn:uuid: format.
-            if (!m_UuidIdentifierValue.isEmpty()) {
-                ncx_resource.SetMainID(m_UuidIdentifierValue);
-            }
-
-            ncx_resource.SaveToDisk();
-	}
-
-	
-        if (m_PackageVersion.startsWith('2')) { 
-            if (ncx_href.isEmpty()) {
-                load_warning = QObject::tr("The OPF file does not contain an NCX file.") + "\n" + 
+    load_warning = QObject::tr("The OPF file does not contain an NCX file.") + "\n" + 
                                " - " +  QObject::tr("Sigil has created a new one for you.");
-            } else {
-                load_warning = QObject::tr("The NCX file is not present in this EPUB.") + "\n" + 
-                               " - " + QObject::tr("Sigil has created a new one for you.");
-            }
-        }
+
+    m_NCXFilePath = QFileInfo(m_OPFFilePath).absolutePath() + "/toc.ncx";
+
+    // Create a new file for the NCX in the *Extracted Folder* Path
+    // We are relying on an identifier being set from the metadata.
+    // It might not have one if the book does not have the urn:uuid: format.
+    NCXResource ncx_resource(m_ExtractedFolderPath, m_NCXFilePath, m_PackageVersion, NULL);
+    ncx_resource.SetEpubVersion(m_PackageVersion);
+    // put it beside the OPF file
+    ncx_resource.FillWithDefaultText(m_PackageVersion, QFileInfo(m_OPFFilePath).absolutePath());
+    if (!m_UuidIdentifierValue.isEmpty()) {
+        ncx_resource.SetMainID(m_UuidIdentifierValue);
     }
+    ncx_resource.SaveToDisk();
+
+    // now add the NCX to our folder
+    QString bookpath = m_NCXFilePath.right(m_NCXFilePath.length() - m_ExtractedFolderPath.length() - 1);
+    m_Book->GetFolderKeeper()->AddNCXToFolder(m_PackageVersion, bookpath);
 
     if (!load_warning.isEmpty()) {
         AddLoadWarning(load_warning);
@@ -893,10 +931,12 @@ void ImportEPUB::LoadInfrastructureFiles()
 }
 
 
-QHash<QString, QString> ImportEPUB::LoadFolderStructure()
+bool ImportEPUB::LoadFolderStructure()
 {
     QList<QString> keys = m_Files.keys();
     int num_files = keys.count();
+    bool success = true;
+
     QFutureSynchronizer<std::tuple<QString, QString>> sync;
 
     for (int i = 0; i < num_files; ++i) {
@@ -911,32 +951,34 @@ QHash<QString, QString> ImportEPUB::LoadFolderStructure()
     sync.waitForFinished();
     QList<QFuture<std::tuple<QString, QString>>> futures = sync.futures();
     int num_futures = futures.count();
-    QHash<QString, QString> updates;
 
     for (int i = 0; i < num_futures; ++i) {
         std::tuple<QString, QString> result = futures.at(i).result();
-        updates[std::get<0>(result)] = std::get<1>(result);
+	if (std::get<0>(result) != std::get<1>(result)) {
+	    qDebug() << "LoadFolderStructure Issue: " << std::get<0>(result) << std::get<1>(result);
+	    success = false;
+	}
     }
 
-    updates.remove(UPDATE_ERROR_STRING);
-    return updates;
+    return success;
 }
 
 
 std::tuple<QString, QString> ImportEPUB::LoadOneFile(const QString &path, const QString &mimetype)
 {
+    // Use opf relative href to create the book path (currentpath) for this file
     QString fullfilepath = QDir::cleanPath(QFileInfo(m_OPFFilePath).absolutePath() + "/" + path);
     QString currentpath = fullfilepath;
     currentpath = currentpath.remove(0,m_ExtractedFolderPath.length()+1);
     try {
-        Resource *resource = m_Book->GetFolderKeeper()->AddContentFileToFolder(fullfilepath, false, mimetype);
+        QString bookpath = currentpath;
+        Resource *resource = m_Book->GetFolderKeeper()->AddContentFileToFolder(fullfilepath, false, mimetype, bookpath);
         if (path == m_NavHref) {
             m_NavResource = resource;
         }
-        resource->SetCurrentBookRelPath(currentpath);
-        QString newpath = "../" + resource->GetRelativePathToOEBPS();
+        QString newpath = resource->GetRelativePath();
         return std::make_tuple(currentpath, newpath);
-    } catch (FileDoesNotExist) {
+    } catch (FileDoesNotExist&) {
         return std::make_tuple(UPDATE_ERROR_STRING, UPDATE_ERROR_STRING);
     }
 }
