@@ -1,6 +1,6 @@
 /************************************************************************
 **
-**  Copyright (C) 2015-2019 Kevin B. Hendricks, Stratford, Ontario Canada
+**  Copyright (C) 2015-2020 Kevin B. Hendricks, Stratford, Ontario Canada
 **  Copyright (C) 2009-2011 Strahinja Markovic  <strahinja.markovic@gmail.com>
 **
 **  This file is part of Sigil.
@@ -123,7 +123,7 @@ void BookBrowser::SetBook(QSharedPointer<Book> book)
         EmitResourceActivated(m_OPFModel->GetFirstHTMLModelIndex());
     }
     // No exception variable since we don't use it
-    catch (NoHTMLFiles) {
+    catch (NoHTMLFiles&) {
         // Do nothing. No HTML files, no first file opened.
     }
 }
@@ -131,6 +131,7 @@ void BookBrowser::SetBook(QSharedPointer<Book> book)
 void BookBrowser::RefreshCounts()
 {
     int  mainfolder_length = m_Book->GetFolderKeeper()->GetFullPathToMainFolder().length();
+    Q_UNUSED(mainfolder_length);
     for (int i = 0; i < m_OPFModel->invisibleRootItem()->rowCount(); i++) {
         QStandardItem *folder = m_OPFModel->invisibleRootItem()->child(i);
 	QString tooltip;
@@ -275,7 +276,7 @@ Resource *BookBrowser::GetUrlResource(const QUrl &url)
         try {
             Resource *resource = m_Book->GetFolderKeeper()->GetResourceByBookPath(bookpath);
             return resource;
-        } catch (ResourceDoesNotExist) {
+        } catch (ResourceDoesNotExist&) {
             Utility::DisplayStdErrorDialog(tr("The file \"%1\" does not exist.").arg(bookpath));
         }
     }
@@ -299,15 +300,17 @@ void BookBrowser::OpenContextMenu(const QPoint &point)
         return;
     }
 
-    m_ContextMenu->exec(m_TreeView->viewport()->mapToGlobal(point));
-    m_ContextMenu->clear();
-    // Ensure any actions with keyboard shortcuts that might have temporarily been
-    // disabled are enabled again to let the shortcut work outside the menu.
-    m_Delete->setEnabled(true);
-    m_Merge->setEnabled(true);
-    m_Rename->setEnabled(true);
-    m_RERename->setEnabled(true);
-    m_Move->setEnabled(true);
+    m_ContextMenu->exec(m_TreeView->viewport()->mapToGlobal(point)); 
+    if (!m_ContextMenu.isNull()) {
+        m_ContextMenu->clear();
+        // Ensure any actions with keyboard shortcuts that might have temporarily been
+        // disabled are enabled again to let the shortcut work outside the menu.
+        m_Delete->setEnabled(true);
+        m_Merge->setEnabled(true);
+        m_Rename->setEnabled(true);
+        m_RERename->setEnabled(true);
+        m_Move->setEnabled(true);
+    }
 }
 
 QList <Resource *> BookBrowser::ValidSelectedHTMLResources()
@@ -643,6 +646,8 @@ QStringList BookBrowser::AddExisting(bool only_multimedia, bool only_images)
         progress.setMinimumDuration(PROGRESS_BAR_MINIMUM_DURATION);
         progress.setValue(progress_value);
     }
+    bool yes_to_all = false;
+    bool no_to_all = false;
     foreach(QString filepath, filepaths) {
         if (file_count > 1) {
             // Set progress value and ensure dialog has time to display when doing extensive updates
@@ -681,14 +686,28 @@ QStringList BookBrowser::AddExisting(bool only_multimedia, bool only_images)
                 SVG_EXTENSIONS.contains(QFileInfo(filepath).suffix().toLower()) ||
                 VIDEO_EXTENSIONS.contains(QFileInfo(filepath).suffix().toLower()) ||
                 AUDIO_EXTENSIONS.contains(QFileInfo(filepath).suffix().toLower())) {
-                QMessageBox::StandardButton button_pressed;
-                button_pressed = QMessageBox::warning(this,
-                                                      tr("Sigil"), tr("The multimedia file \"%1\" already exists in the book.\n\nOK to replace?").arg(filename),
-                                                      QMessageBox::Ok | QMessageBox::Cancel);
+		bool do_replacement = false;
+		if (yes_to_all) do_replacement = true;
+		if (no_to_all) do_replacement = false;
+		if (!yes_to_all && !no_to_all) {
+                   QMessageBox::StandardButton button_pressed;
+                   button_pressed = QMessageBox::warning(this, tr("Sigil"), 
+			tr("The multimedia file \"%1\" already exists in the book.\n\nOK to replace?").arg(filename),
+				  QMessageBox::Yes | QMessageBox::YesToAll | QMessageBox::No | QMessageBox::NoToAll);
 
-                if (button_pressed != QMessageBox::Ok) {
-                    continue;
-                }
+		   if (button_pressed == QMessageBox::YesToAll) {
+		       yes_to_all = true;
+		       do_replacement = true;
+		   }
+		   if (button_pressed == QMessageBox::NoToAll) {
+		       no_to_all = true;
+		       do_replacement = false;
+		   }
+		   if (button_pressed == QMessageBox::Yes) do_replacement = true;
+		   if (button_pressed == QMessageBox::No) do_replacement = false;
+		}
+
+		if (!do_replacement) continue; 
 
                 try {
                     Resource *old_resource = m_Book->GetFolderKeeper()->GetResourceByBookPath(existing_book_path);
@@ -697,7 +716,7 @@ QStringList BookBrowser::AddExisting(bool only_multimedia, bool only_images)
 		        CoverImageSemanticsSet = m_Book->GetOPF()->IsCoverImage(image_resource);
 		    }
                     old_resource->Delete();
-                } catch (ResourceDoesNotExist) {
+                } catch (ResourceDoesNotExist&) {
                     Utility::DisplayStdErrorDialog(tr("Unable to delete or replace file \"%1\".").arg(filename)
                     );
                     continue;
@@ -726,6 +745,7 @@ QStringList BookBrowser::AddExisting(bool only_multimedia, bool only_images)
             bool extract_metadata = false;
             html_import.GetBook(extract_metadata);
 	    QStringList importedbookpaths = html_import.GetAddedBookPaths();
+            qDebug() << importedbookpaths;
             Resource *added_resource = m_Book->GetFolderKeeper()->GetResourceByBookPath(importedbookpaths.at(0));
             HTMLResource *added_html_resource = qobject_cast<HTMLResource *>(added_resource);
 	    added_book_paths.append(importedbookpaths);
@@ -974,6 +994,7 @@ void BookBrowser::Rename()
     }
 
     Resource::ResourceType resource_type = resources.first()->Type();
+    Q_UNUSED(resource_type);
 
     if (resources.count() == 1) {
         // Save the resource so it can be re-selected
@@ -994,7 +1015,8 @@ void BookBrowser::REXRename()
     }
 
     Resource::ResourceType resource_type = resources.first()->Type();
-
+    Q_UNUSED(resource_type);
+    
     QString retext;
     QString replacetext;
     int trycnt = 4;
@@ -1650,7 +1672,7 @@ void BookBrowser::SetupTreeView()
 {
     m_TreeView->setEditTriggers(QAbstractItemView::EditKeyPressed);
     m_TreeView->setSortingEnabled(false);
-    m_TreeView->sortByColumn(-1);
+    m_TreeView->sortByColumn(-1, Qt::AscendingOrder);
     m_TreeView->setUniformRowHeights(true);
     m_TreeView->setDragEnabled(true);
     m_TreeView->setAcceptDrops(false);
