@@ -106,7 +106,7 @@ public:
 	       const QString version = QString(),
 	       bool is_internal = false, 
 	       QWidget *parent = 0, 
-	       Qt::WindowFlags flags = 0);
+	       Qt::WindowFlags flags = Qt::WindowFlags());
     ~MainWindow();
 
     void maybe_fixup_dockwidget_geometry(QDockWidget * widget);
@@ -204,7 +204,10 @@ public:
     QString GetCurrentFilePath();
 
     void DebugCurrentWidgetSizes();
-    
+
+    QString GetMathJaxFolder() { return m_mathjaxfolder; };
+
+
 public slots:
 
     void OpenUrl(const QUrl &url);
@@ -242,6 +245,11 @@ public slots:
 
     void launchExternalXEditor();
 
+    void RepoCommit();
+    void RepoCheckout(QString bookid="", QString destpath="", QString filename="", bool loadnow=true);
+    void RepoDiff(QString bookid="");
+    void RepoManage();
+
     void StandardizeEpub();
 
     void CreateEpubLayout();
@@ -628,7 +636,8 @@ private slots:
     void DeleteReportsStyles(QList<BookReports::StyleData *> reports_styles_to_delete);
 
     void DeleteFilenames(QStringList files_to_delete);
-    void OpenFile(QString file_shortpathname, int line = -1);
+
+    void OpenFile(QString file_bookpath, int line = -1, int position = -1);
 
     void UpdateClipsUI();
 
@@ -639,6 +648,7 @@ private slots:
     void unloadPluginsMenu();
 
 private:
+    void createJumpList();
     void updateToolTipsOnPluginIcons();
     void UpdateClipButton(QAction *ui_action);
     void InsertFiles(const QStringList &selected_images);
@@ -742,7 +752,7 @@ private:
      *
      * @param fullfilepath The path to the currently edited file.
      */
-    void UpdateUiWithCurrentFile(const QString &fullfilepath);
+    void UpdateUiWithCurrentFile(const QString &fullfilepath, bool just_name = false);
 
     /**
      * Selects the appropriate entry in the heading combo box
@@ -1015,6 +1025,8 @@ private:
      */
     QActionGroup *m_headingActionGroup;
 
+    QString m_mathjaxfolder;
+
     /**
      * Holds all the widgets Qt Designer created for us.
      */
