@@ -39,6 +39,7 @@ static QString KEY_DEFAULT_METADATA_LANGUAGE = SETTINGS_GROUP + "/" + "default_m
 static QString KEY_UI_LANGUAGE = SETTINGS_GROUP + "/" + "ui_language";
 static QString KEY_UI_FONT = SETTINGS_GROUP + "/" + "ui_font";
 static QString KEY_ORIGINAL_UI_FONT = SETTINGS_GROUP + "/" + "original_ui_font";
+static QString KEY_UI_ICON_THEME = SETTINGS_GROUP + "/" + "ui_icon_theme";
 static QString KEY_DRAG_DISTANCE_TWEAK = SETTINGS_GROUP + "/" + "drag_distance_tweak";
 static QString KEY_ZOOM_IMAGE = SETTINGS_GROUP + "/" + "zoom_image";
 static QString KEY_ZOOM_TEXT = SETTINGS_GROUP + "/" + "zoom_text";
@@ -47,6 +48,7 @@ static QString KEY_ZOOM_PREVIEW = SETTINGS_GROUP + "/" + "zoom_preview";
 static QString KEY_ZOOM_INSPECTOR = SETTINGS_GROUP + "/" + "zoom_inspector";
 static QString KEY_RENAME_TEMPLATE = SETTINGS_GROUP + "/" + "rename_template";
 static QString KEY_DICTIONARY_NAME = SETTINGS_GROUP + "/" + "dictionary_name";
+static QString KEY_SECONDARY_DICTIONARY_NAME = SETTINGS_GROUP + "/" + "secondary_dictionary_name";
 static QString KEY_SPELL_CHECK = SETTINGS_GROUP + "/" + "spell_check";
 static QString KEY_SPELL_CHECK_NUMBERS = SETTINGS_GROUP + "/" + "spell_check_numbers";
 static QString KEY_DEFAULT_USER_DICTIONARY = SETTINGS_GROUP + "/" + "user_dictionary_name";
@@ -158,6 +160,12 @@ QString SettingsStore::originalUIFont()
     return value(KEY_ORIGINAL_UI_FONT, "").toString();
 }
 
+QString SettingsStore::uiIconTheme()
+{
+    clearSettingsGroup();
+    return value(KEY_UI_ICON_THEME, "main").toString();
+}
+
 int SettingsStore::uiDragDistanceTweak()
 {
     clearSettingsGroup();
@@ -212,6 +220,12 @@ QString SettingsStore::dictionary()
     return value(KEY_DICTIONARY_NAME, "en_US").toString();
 }
 
+QString SettingsStore::secondary_dictionary()
+{
+    clearSettingsGroup();
+    return value(KEY_SECONDARY_DICTIONARY_NAME, "").toString();
+}
+
 QStringList SettingsStore::enabledUserDictionaries()
 {
     clearSettingsGroup();
@@ -221,7 +235,7 @@ QStringList SettingsStore::enabledUserDictionaries()
 bool SettingsStore::spellCheck()
 {
     clearSettingsGroup();
-    return static_cast<bool>(value(KEY_SPELL_CHECK, true).toBool());
+    return static_cast<bool>(value(KEY_SPELL_CHECK, false).toBool());
 }
 
 bool SettingsStore::spellCheckNumbers()
@@ -480,6 +494,12 @@ void SettingsStore::setOriginalUIFont(const QString &font_data)
     setValue(KEY_ORIGINAL_UI_FONT, font_data);
 }
 
+void SettingsStore::setUIIconTheme(const QString &iconthemename)
+{
+    clearSettingsGroup();
+    setValue(KEY_UI_ICON_THEME, iconthemename);
+}
+
 void SettingsStore::setUiDragDistanceTweak(int tweak)
 {
     clearSettingsGroup();
@@ -528,6 +548,12 @@ void SettingsStore::setDictionary(const QString &name)
     setValue(KEY_DICTIONARY_NAME, name);
 }
 
+void SettingsStore::setSecondaryDictionary(const QString &name)
+{
+    clearSettingsGroup();
+    setValue(KEY_SECONDARY_DICTIONARY_NAME, name);
+}
+
 void SettingsStore::setEnabledUserDictionaries(const QStringList names)
 {
     clearSettingsGroup();
@@ -803,6 +829,7 @@ void SettingsStore::clearAppearanceSettings()
     remove(KEY_SHOWFULLPATH_ON);
     remove(KEY_HIGHDPI_SETTING);
     remove(KEY_UI_FONT);
+    remove(KEY_UI_ICON_THEME);
     remove(KEY_DRAG_DISTANCE_TWEAK);
     remove(KEY_PREVIEW_DARK_IN_DM);
     ;
