@@ -60,6 +60,7 @@ public:
 
     QString uiFont();
     QString originalUIFont();
+    QString uiIconTheme();
 
     int uiDragDistanceTweak();
 
@@ -92,6 +93,9 @@ public:
 
     QStringList enabledUserDictionaries();
 
+    QString secondary_dictionary();
+
+
     /**
      * Get the list of entities/code pairs to preserve
      */
@@ -258,6 +262,8 @@ public slots:
 
     void setOriginalUIFont(const QString &font_data);
 
+    void setUIIconTheme(const QString &iconthemename);
+
     void setUiDragDistanceTweak(int tweak);
 
     /**
@@ -289,6 +295,8 @@ public slots:
 
     void setEnabledUserDictionaries(const QStringList name);
 
+    void setSecondaryDictionary(const QString &name);
+
     /**
      * Set the list of paired code, entity strings to preserve.
      */
