@@ -59,6 +59,7 @@
 #include <QFileInfo>
 #include <QCollator>
 #include <QMenu>
+#include <QVector>
 #include <QDebug>
 
 #include "sigil_constants.h"
@@ -68,6 +69,8 @@
 #include "Misc/SleepFunctions.h"
 #include "MainUI/MainApplication.h"
 
+static const QString URL_SAFE = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789_.-/~";
+
 static const QString DARK_STYLE =
     "<style>:root { background-color: %1; color: %2; } ::-webkit-scrollbar { display: none; }</style>"
     "<link rel=\"stylesheet\" type=\"text/css\" href=\"%3\" />";
@@ -272,12 +275,22 @@ bool Utility::IsMixedCase(const QString &string)
     return false;
 }
 
+// Returns a substring from a specified QStringRef;
+// the characters included are in the interval:
+// [ start_index, end_index >
+QString Utility::Substring(int start_index, int end_index, const QStringRef &string)
+{
+    return string.mid(start_index, end_index - start_index).toString();
+}
+
+
 // Returns a substring of a specified string;
 // the characters included are in the interval:
 // [ start_index, end_index >
 QString Utility::Substring(int start_index, int end_index, const QString &string)
 {
     return string.mid(start_index, end_index - start_index);
+
 }
 
 // Returns a substring of a specified string;
@@ -570,27 +583,82 @@ QString Utility::DecodeXML(const QString &text)
 
 QString Utility::EncodeXML(const QString &text)
 {
-    QString newtext(text);
+    QString newtext = Utility::DecodeXML(text);
     return newtext.toHtmlEscaped();
 }
 
+
+
+// From the IRI spec rfc3987
+// iunreserved    = ALPHA / DIGIT / "-" / "." / "_" / "~" / ucschar
+// 
+//    ucschar        = %xA0-D7FF / %xF900-FDCF / %xFDF0-FFEF
+//                   / %x10000-1FFFD / %x20000-2FFFD / %x30000-3FFFD
+//                   / %x40000-4FFFD / %x50000-5FFFD / %x60000-6FFFD
+//                   / %x70000-7FFFD / %x80000-8FFFD / %x90000-9FFFD
+//                   / %xA0000-AFFFD / %xB0000-BFFFD / %xC0000-CFFFD
+//                   / %xD0000-DFFFD / %xE1000-EFFFD
+// But currently nothing *after* the 0x30000 plane is even defined
+
+bool Utility::NeedToPercentEncode(uint32_t cp)
+{
+    // sequence matters for both correctness and speed
+    if (cp < 128) {
+        if (URL_SAFE.contains(QChar(cp))) return false;
+        return true;
+    }
+    if (cp < 0xA0) return true;
+    if (cp <= 0xD7FF) return false;
+    if (cp < 0xF900) return true;
+    if (cp <= 0xFDCF) return false;
+    if (cp < 0xFDF0) return true;
+    if (cp <= 0xFFEF) return false;
+    if (cp < 0x10000) return true;
+    if (cp <= 0x1FFFD) return false;
+    if (cp < 0x20000) return true;
+    if (cp <= 0x2FFFD) return false;
+    if (cp < 0x30000) return true;
+    if (cp <= 0x3FFFD) return false;
+    return true;
+}
+
+// this is meant to work on paths, not paths and fragments and schemes
+// therefore do not leave # chars unencoded
 QString Utility::URLEncodePath(const QString &path)
 {
-    QString newpath = path;
-    QUrl href = QUrl(newpath);
-    QString scheme = href.scheme();
-    if (!scheme.isEmpty()) {
-        scheme = scheme + "://";
-        newpath.remove(0, scheme.length());
-    }
-
     // some very poorly written software uses xml escaping of the 
     // "&" instead of url encoding when building hrefs
     // So run xmldecode first to convert them to normal characters before 
     // url encoding them
-    newpath = DecodeXML(newpath);
-    QByteArray encoded_url = QUrl::toPercentEncoding(newpath, QByteArray("/#"));
-    return scheme + QString::fromUtf8(encoded_url.constData(), encoded_url.count());
+    QString newpath = DecodeXML(path);
+
+    // then undo any existing url encoding
+    newpath = URLDecodePath(newpath);
+
+    QString result = "";
+    QVector<uint32_t> codepoints = newpath.toUcs4();
+    for (int i = 0; i < codepoints.size(); i++) {
+        uint32_t cp = codepoints.at(i);
+        QString s = QString::fromUcs4(&cp, 1);
+        if (NeedToPercentEncode(cp)) {
+            QByteArray b = s.toUtf8();
+            for (int j = 0; j < b.size(); j++) {
+                uint8_t bval = b.at(j);
+                QString val = QString::number(bval,16);
+                val = val.toUpper();
+                if (val.size() == 1) val.prepend("0");
+                val.prepend("%");
+                result.append(val);
+            }
+        } else {
+            result.append(s);
+        }
+    }
+    // qDebug() << "In Utility URLEncodePath: " << result;
+    // Previously was:
+    // encoded_url = QUrl::toPercentEncoding(newpath, QByteArray("/"), QByteArray("#"));
+    // encoded_path = scheme + QString::fromUtf8(encoded_url.constData(), encoded_url.count());
+    return result;
 }
 
 
@@ -720,6 +788,7 @@ bool Utility::has_non_ascii_chars(const QString &str)
 
 bool Utility::use_filename_warning(const QString &filename)
 {
+#if 0
     const QString uri_delimiters = ":/?#[]@";
     foreach(QChar c, uri_delimiters) {
         if (filename.contains(c)) {
@@ -732,6 +801,7 @@ bool Utility::use_filename_warning(const QString &filename)
             return false;
         }
     }
+#endif
     if (has_non_ascii_chars(filename)) {
         return QMessageBox::Apply == QMessageBox::warning(QApplication::activeWindow(),
                 tr("Sigil"),
@@ -1117,22 +1187,43 @@ QString Utility::buildRelativePath(const QString &from_file_bkpath, const QStrin
 
     // convert start_file_bkpath to start_dir by stripping off existing filename component
     return relativePath(to_file_bkpath, startingDir(from_file_bkpath));
-}   
+}
 
-std::pair<QString, QString> Utility::parseHREF(const QString &relative_href)
+// return fully decoded path and fragment (if any) from a raw relative href string.
+// any fragment will start with '#', use QUrl to handle parsing and Percent Decoding
+std::pair<QString, QString> Utility::parseRelativeHREF(const QString &relative_href)
 {
-    QString fragment;
-    QString attpath = relative_href;
-    int fragpos = attpath.lastIndexOf("#");
-    // fragment will include any # if one exists
-    if (fragpos != -1) {
-        fragment = attpath.mid(fragpos, -1);
-        attpath = attpath.mid(0, fragpos);
+    QUrl href(relative_href);
+    Q_ASSERT(href.isRelative());
+    Q_ASSERT(!href.hasQuery());
+    QString attpath = href.path();
+    QString fragment = href.fragment();
+    // fragment will include any # if fragment exists
+    if (relative_href.indexOf("#") != -1) {
+        fragment = "#" + fragment;
     }
     if (attpath.startsWith("./")) attpath = attpath.mid(2,-1);
     return std::make_pair(attpath, fragment);
 }
 
+// return a url encoded string for given decoded path and fragment (if any)
+// Note: Any fragment will start with a "#" ! to allow links to root as just "#"
+QString Utility::buildRelativeHREF(const QString &apath, const QString &afrag)
+{
+    QString newhref = URLEncodePath(apath);
+    QString id = afrag;
+    if (!id.isEmpty()) {
+        if (id.startsWith("#")) {
+            id = id.mid(1, -1);
+        } else {
+	    qDebug() << "Warning: buildRelativeHREF has fragment that does not start with #" << afrag;
+        }
+	// technically fragments should be percent encoded if needed
+	id = URLEncodePath(id);
+	newhref = newhref + "#" + id;
+    }
+    return newhref;
+}
 
 bool Utility::sort_pair_in_reverse(const std::pair<int,QString> &a, const std::pair<int,QString> &b)
 {
