@@ -31,6 +31,7 @@
 #include <QRegularExpression>
 #include <QRegularExpressionMatch>
 #include <QDateTime>
+#include <QDebug>
 
 #include "BookManipulation/CleanSource.h"
 #include "BookManipulation/XhtmlDoc.h"
@@ -46,9 +47,8 @@
 #include "ResourceObjects/OPFResource.h"
 #include "ResourceObjects/OPFParser.h"
 #include "ResourceObjects/NavProcessor.h"
-
-
 #include "sigil_constants.h"
+#include "sigil_exception.h"
 
 static const QString SIGIL_VERSION_META_NAME  = "Sigil version";
 static const QString OPF_XML_NAMESPACE        = "http://www.idpf.org/2007/opf";
@@ -176,11 +176,26 @@ QString OPFResource::GetText() const
 
 void OPFResource::SetText(const QString &text)
 {
+    emit TextChanging();
     QWriteLocker locker(&GetLock());
     QString source = ValidatePackageVersion(text);
     TextResource::SetText(source);
 }
 
+
+bool OPFResource::LoadFromDisk()
+{
+    try {
+        const QString &text = Utility::ReadUnicodeTextFile(GetFullPath());
+	SetText(text);
+        emit LoadedFromDisk();
+	return true;
+    } catch (CannotOpenFile&) {
+        //
+    }
+    return false;
+}
+
 QList<Resource*> OPFResource::GetSpineOrderResources( const QList<Resource *> &resources)
 {
     QReadLocker locker(&GetLock());
@@ -320,6 +335,10 @@ void OPFResource::EnsureUUIDIdentifierPresent()
         }
     }
     QString uuid = Utility::CreateUUID();
+    // add in the proper identifier type prefix
+    if (!uuid.startsWith("urn:uuid:")) {
+	uuid = "urn:uuid:" + uuid;
+    }
     WriteIdentifier("UUID", uuid, p);
     UpdateText(p);
 }
@@ -337,7 +356,7 @@ QString OPFResource::AddNCXItem(const QString &ncx_path, QString id)
     int n = p.m_manifest.count();
     ManifestEntry me;
     me.m_id = GetUniqueID(id, p);
-    me.m_href = ncx_rel_path;
+    me.m_href = Utility::URLEncodePath(ncx_rel_path);
     me.m_mtype = "application/x-dtbncx+xml";
     p.m_manifest.append(me);
     p.m_idpos[me.m_id] = n;
@@ -382,7 +401,7 @@ void OPFResource::UpdateNCXLocationInManifest(const NCXResource *ncx)
     if (pos > -1) {
         ManifestEntry me = p.m_manifest.at(pos);
         QString href = me.m_href;
-	QString new_href = GetRelativePathToResource(ncx);
+	QString new_href = Utility::URLEncodePath(GetRelativePathToResource(ncx));
         me.m_href = new_href;
         p.m_manifest.replace(pos, me);
         p.m_hrefpos.remove(href);
@@ -471,20 +490,30 @@ QStringList OPFResource::GetSpineOrderBookPaths() const
         QString idref = sp.m_idref;
         int pos = p.m_idpos.value(idref,-1);
         if (pos > -1) {
-            QString href = p.m_manifest.at(pos).m_href;
-            book_paths_in_reading_order.append(Utility::buildBookPath(href,GetFolder()));
+            QString apath = Utility::URLDecodePath(p.m_manifest.at(pos).m_href);
+            book_paths_in_reading_order.append(Utility::buildBookPath(apath,GetFolder()));
         }
     }
     return book_paths_in_reading_order;
 }
 
+QString OPFResource::GetPrimaryBookTitle() const
+{
+    QString title = "";
+    QStringList titles = GetDCMetadataValues("dc:title");
+    if (!titles.isEmpty()) {
+	title = titles.at(0);
+    }
+    return title;
+}
+
 QString OPFResource::GetPrimaryBookLanguage() const
 {
     SettingsStore settings;
     QString lang = settings.defaultMetadataLang();
-    QList<QVariant> languages = GetDCMetadataValues("dc:language");
+    QStringList languages = GetDCMetadataValues("dc:language");
     if (!languages.isEmpty()) {
-        lang = languages.at(0).toString();
+        lang = languages.at(0);
     }
     return lang;
 }
@@ -506,15 +535,15 @@ QList<MetaEntry> OPFResource::GetDCMetadata() const
 }
 
 
-QList<QVariant> OPFResource::GetDCMetadataValues(QString text) const
+QStringList OPFResource::GetDCMetadataValues(QString text) const
 {
-    QList<QVariant> values;
+    QStringList metavalues;
     foreach(MetaEntry meta, GetDCMetadata()) {
         if (meta.m_name == text) {
-            values.append(meta.m_content);
+            metavalues.append(meta.m_content);
         }
     }
-    return values;
+    return metavalues;
 }
 
 
@@ -543,7 +572,7 @@ void OPFResource::AddResource(const Resource *resource)
     p.parse(source);
     ManifestEntry me;
     me.m_id = GetUniqueID(GetValidID(resource->Filename()),p);
-    me.m_href = GetRelativePathToResource(resource);
+    me.m_href = Utility::URLEncodePath(GetRelativePathToResource(resource));
     me.m_mtype = GetResourceMimetype(resource);
     // Argh! If this is an new blank resource - it will have no content yet
     // so trying to parse it here to check for manifest properties is a mistake
@@ -649,7 +678,7 @@ void OPFResource::RemoveResource(const Resource *resource)
     OPFParser p;
     p.parse(source);
     if (p.m_manifest.isEmpty()) return;
-    QString href = GetRelativePathToResource(resource);
+    QString href = Utility::URLEncodePath(GetRelativePathToResource(resource));
     int pos = p.m_hrefpos.value(href, -1);
     QString item_id = "";
 
@@ -736,7 +765,7 @@ QString OPFResource::GetGuideSemanticCodeForResource(const Resource *resource, c
 
 int OPFResource::GetGuideReferenceForResourcePos(const Resource *resource, const OPFParser &p) const
 {
-  QString href_to_resource_from_opf = GetRelativePathToResource(resource);
+  QString href_to_resource_from_opf = Utility::URLEncodePath(GetRelativePathToResource(resource));
     for (int i=0; i < p.m_guide.count(); ++i) {
         GuideEntry ge = p.m_guide.at(i);
         QString href = ge.m_href;
@@ -795,7 +824,7 @@ void OPFResource::SetGuideSemanticCodeForResource(QString code, const Resource *
         GuideEntry ge;
         ge.m_type = code;
         ge.m_title = title;
-        ge.m_href = GetRelativePathToResource(resource);
+        ge.m_href = Utility::URLEncodePath(GetRelativePathToResource(resource));
         p.m_guide.append(ge);
     }
 }
@@ -828,7 +857,8 @@ QHash <QString, QString>  OPFResource::GetSemanticCodeForPaths()
   foreach(GuideEntry ge, p.m_guide) {
     QString href = ge.m_href;
     QStringList parts = href.split('#', QString::KeepEmptyParts);
-    QString bkpath = Utility::buildBookPath(parts.at(0),GetFolder());
+    QString apath = Utility::URLDecodePath(parts.at(0));
+    QString bkpath = Utility::buildBookPath(apath, GetFolder());
     QString gtype = ge.m_type;
     semantic_types[bkpath] = gtype;
   }
@@ -848,7 +878,8 @@ QHash <QString, QString>  OPFResource::GetGuideSemanticNameForPaths()
         QString href = ge.m_href;
         QStringList parts = href.split('#', QString::KeepEmptyParts);
         QString gtype = ge.m_type;
-	QString bkpath = Utility::buildBookPath(parts.at(0),GetFolder());
+        QString apath = Utility::URLDecodePath(parts.at(0));
+	QString bkpath = Utility::buildBookPath(apath, GetFolder());
         semantic_types[bkpath] = GuideItems::instance()->GetName(gtype);
     }
 
@@ -858,9 +889,9 @@ QHash <QString, QString>  OPFResource::GetGuideSemanticNameForPaths()
         MetaEntry me = p.m_metadata.at(pos);
         QString cover_id = me.m_atts.value(QString("content"),QString(""));
         ManifestEntry man = p.m_manifest.at(p.m_idpos[cover_id]);
-        QString href = man.m_href;
-	href = Utility::buildBookPath(href, GetFolder());
-        semantic_types[href] = GuideItems::instance()->GetName("cover");
+        QString apath = Utility::URLDecodePath(man.m_href);
+	QString bkpath = Utility::buildBookPath(apath, GetFolder());
+        semantic_types[bkpath] = GuideItems::instance()->GetName("cover");
     }
     return semantic_types;
 }
@@ -943,7 +974,7 @@ void OPFResource::ResourceRenamed(const Resource *resource, QString old_full_pat
     p.parse(source);
     // first convert old_full_path to old_bkpath
     QString old_bkpath = old_full_path.right(old_full_path.length() - GetFullPathToBookFolder().length() - 1);
-    QString old_href = Utility::buildRelativePath(GetRelativePath(), old_bkpath);
+    QString old_href = Utility::URLEncodePath(Utility::buildRelativePath(GetRelativePath(), old_bkpath));
     QString old_id;
     QString new_id;
     for (int i=0; i < p.m_manifest.count(); ++i) {
@@ -951,7 +982,7 @@ void OPFResource::ResourceRenamed(const Resource *resource, QString old_full_pat
         if (href == old_href) {
             ManifestEntry me = p.m_manifest.at(i);
             QString old_me_href = me.m_href;
-            me.m_href = GetRelativePathToResource(resource);
+            me.m_href = Utility::URLEncodePath(GetRelativePathToResource(resource));
             old_id = me.m_id;
             p.m_idpos.remove(old_id);
             new_id = GetUniqueID(GetValidID(resource->Filename()),p);
@@ -1001,14 +1032,14 @@ void OPFResource::ResourceMoved(const Resource *resource, QString old_full_path)
     p.parse(source);
     // first convert old_full_path to old_bkpath
     QString old_bkpath = old_full_path.right(old_full_path.length() - GetFullPathToBookFolder().length() - 1);
-    QString old_href = Utility::buildRelativePath(GetRelativePath(), old_bkpath);
+    QString old_href = Utility::URLEncodePath(Utility::buildRelativePath(GetRelativePath(), old_bkpath));
     // a move should not impact the id so leave the old unique manifest id unchanged
     for (int i=0; i < p.m_manifest.count(); ++i) {
         QString href = p.m_manifest.at(i).m_href;
         if (href == old_href) {
             ManifestEntry me = p.m_manifest.at(i);
             QString old_me_href = me.m_href;
-            me.m_href = GetRelativePathToResource(resource);
+            me.m_href = Utility::URLEncodePath(GetRelativePathToResource(resource));
             p.m_idpos[me.m_id] = i;
             p.m_hrefpos.remove(old_me_href);
             p.m_hrefpos[me.m_href] = i;
@@ -1055,7 +1086,7 @@ QHash<QString, Resource*>OPFResource::GetManifestIDResourceMapping(const QList<R
 {
     QHash<QString, Resource*> id_mapping;
     foreach(Resource * resource, resources) {
-        QString href_path = GetRelativePathToResource(resource);
+        QString href_path = Utility::URLEncodePath(GetRelativePathToResource(resource));
         int pos = p.m_hrefpos.value(href_path,-1);
         if (pos > -1) { 
 	    id_mapping[ p.m_manifest.at(pos).m_id ] = resource;
@@ -1067,11 +1098,12 @@ QHash<QString, Resource*>OPFResource::GetManifestIDResourceMapping(const QList<R
 
 QString OPFResource::GetResourceManifestID(const Resource *resource, const OPFParser& p) const
 {
-    QString href_path = GetRelativePathToResource(resource);
+    QString href_path = Utility::URLEncodePath(GetRelativePathToResource(resource));
     int pos = p.m_hrefpos.value(href_path,-1);
     if (pos > -1) { 
         return QString(p.m_manifest.at(pos).m_id); 
     }
+    qDebug() << "GetResourceMainfestID returning null" << href_path;
     return QString();
 }
 
@@ -1080,11 +1112,17 @@ QHash<Resource *, QString> OPFResource::GetResourceManifestIDMapping(const QList
                                                                      const OPFParser& p)
 {
     QHash<Resource *, QString> id_mapping;
+    QStringList keys = p.m_hrefpos.keys();
     foreach(Resource * resource, resources) {
-      QString href_path = GetRelativePathToResource(resource);
-        int pos = p.m_hrefpos.value(href_path,-1);
-        if (pos > -1) { 
-            id_mapping[ resource ] = p.m_manifest.at(pos).m_id;
+        QString href_path = Utility::URLEncodePath(GetRelativePathToResource(resource));
+        // empty relative path from the OPF is the OPF which will not have a manifest entry
+        if (!href_path.isEmpty()) {
+            int pos = p.m_hrefpos.value(href_path,-1);
+            if (pos > -1) { 
+                id_mapping[ resource ] = p.m_manifest.at(pos).m_id;
+            } else {
+                qDebug() << "GetResourceMainifestIDMapping no map for" << href_path;
+            }
         }
     }
     return id_mapping;
@@ -1127,13 +1165,19 @@ void OPFResource::WriteIdentifier(const QString &metaname, const QString &metava
     if (pos > -1) {
         MetaEntry me = p.m_metadata.at(pos);
         QString scheme = me.m_atts.value(QString("scheme"),QString(""));
+	// epub3 no longer uses the scheme attribute
+	if (scheme.isEmpty() && me.m_content.startsWith("urn:uuid:")) scheme="UUID";
         if ((metavalue == me.m_content) && (metaname == scheme)) {
             return;
         }
     }
+    QString epubversion = GetEpubVersion();
     MetaEntry me;
     me.m_name = QString("dc:identifier");
-    me.m_atts[QString("opf:scheme")] = metaname;
+    // under the latest epub3 spec "scheme" is no longer an allowed attribute of dc:identifier
+    if (epubversion.startsWith('2')) {
+	me.m_atts[QString("opf:scheme")] = metaname;
+    }
     if (metaname.toLower() == "uuid" && !metavalue.contains("urn:uuid:")) {
         me.m_content = QString("urn:uuid:")  + metavalue;
     } else {
@@ -1142,8 +1186,13 @@ void OPFResource::WriteIdentifier(const QString &metaname, const QString &metava
     p.m_metadata.append(me);
 }
 
-void OPFResource::AddModificationDateMeta()
+QString OPFResource::AddModificationDateMeta()
 {
+    QString datetime;
+    QDateTime local(QDateTime::currentDateTime());
+    local.setTimeSpec(Qt::UTC);
+    datetime = local.toString(Qt::ISODate);
+
     QWriteLocker locker(&GetLock());
     QString source = CleanSource::ProcessXML(GetText(),"application/oebps-package+xml");
     OPFParser p;
@@ -1153,9 +1202,6 @@ void OPFResource::AddModificationDateMeta()
     if (epubversion.startsWith('3')) {
 
         // epub 3 set dcterms:modified date time in ISO 8601 format
-        QDateTime local(QDateTime::currentDateTime());
-        local.setTimeSpec(Qt::UTC);
-        QString datetime = local.toString(Qt::ISODate);
         // if an entry exists, update it
         for (int i=0; i < p.m_metadata.count(); ++i) {
             MetaEntry me = p.m_metadata.at(i);
@@ -1165,7 +1211,7 @@ void OPFResource::AddModificationDateMeta()
                     me.m_content = datetime;
                     p.m_metadata.replace(i, me);
                     UpdateText(p);
-                    return;
+                    return datetime;
                 }
             }
         }
@@ -1176,7 +1222,7 @@ void OPFResource::AddModificationDateMeta()
         me.m_atts["property"]="dcterms:modified";
         p.m_metadata.append(me);
         UpdateText(p);
-        return;
+        return datetime;
     }   
     // epub 2 version 
     QString date;
@@ -1194,7 +1240,7 @@ void OPFResource::AddModificationDateMeta()
                 me.m_content = date;
                 p.m_metadata.replace(i, me);
                 UpdateText(p);
-                return;
+                return datetime;
             }
             
         }
@@ -1207,6 +1253,7 @@ void OPFResource::AddModificationDateMeta()
     me.m_atts[QString("opf:event")] = QString("modification");
     p.m_metadata.append(me);
     UpdateText(p);
+    return datetime;
 }
 
 
@@ -1304,7 +1351,7 @@ void OPFResource::UpdateManifestProperties(const QList<Resource*> resources)
         const HTMLResource* html_resource = static_cast<const HTMLResource *>(resource);
         // do not overwrite the nav property, it must stay no matter what
         if (html_resource != m_NavResource) {
-	    QString href = GetRelativePathToResource(html_resource);
+            QString href = Utility::URLEncodePath(GetRelativePathToResource(html_resource));
             int pos = p.m_hrefpos.value(href, -1);
             if ((pos >= 0) && (pos < p.m_manifest.count())) {
                 ManifestEntry me = p.m_manifest.at(pos);
@@ -1347,7 +1394,7 @@ QString OPFResource::GetManifestPropertiesForResource(const Resource * resource)
     if (!p.m_package.m_version.startsWith("3")) {
         return properties;
     }
-    QString href = GetRelativePathToResource(resource);
+    QString href = Utility::URLEncodePath(GetRelativePathToResource(resource));
     int pos = p.m_hrefpos.value(href, -1);
     if ((pos >= 0) && (pos < p.m_manifest.count())) {
         ManifestEntry me = p.m_manifest.at(pos);
@@ -1369,11 +1416,11 @@ QHash <QString, QString>  OPFResource::GetManifestPropertiesForPaths()
     OPFParser p;
     p.parse(source);
     foreach(ManifestEntry me, p.m_manifest) {
-        QString href = me.m_href;
+        QString apath = Utility::URLDecodePath(me.m_href);
         if (me.m_atts.contains("properties")){
             QString properties = me.m_atts["properties"];
-	    href = Utility::buildBookPath(href,GetFolder());
-            manifest_properties_all[href] = properties;
+	    apath = Utility::buildBookPath(apath, GetFolder());
+            manifest_properties_all[apath] = properties;
         }
     }
     return manifest_properties_all;
@@ -1395,7 +1442,7 @@ void OPFResource::SetNavResource(HTMLResource * nav_resource)
         QString source = CleanSource::ProcessXML(GetText(),"application/oebps-package+xml");
         OPFParser p;
         p.parse(source);
-        QString href = GetRelativePathToResource(m_NavResource);
+        QString href = Utility::URLEncodePath(GetRelativePathToResource(m_NavResource));
         int pos = p.m_hrefpos.value(href, -1);
         if ((pos >= 0) && (pos < p.m_manifest.count())) {
             ManifestEntry me = p.m_manifest.at(pos);
@@ -1413,7 +1460,7 @@ void OPFResource::SetItemRefLinear(Resource * resource, bool linear)
     QString source = CleanSource::ProcessXML(GetText(),"application/oebps-package+xml");
     OPFParser p;
     p.parse(source);
-    QString resource_href_path = GetRelativePathToResource(resource);
+    QString resource_href_path = Utility::URLEncodePath(GetRelativePathToResource(resource));
     int pos = p.m_hrefpos.value(resource_href_path, -1);
     QString item_id = "";
     if (pos > -1) {
