@@ -2,7 +2,8 @@
 # -*- coding: utf-8 -*-
 # vim:ts=4:sw=4:softtabstop=4:smarttab:expandtab
 
-# Copyright (c) 2014 Kevin B. Hendricks, John Schember, and Doug Massay
+# Copyright (c) 2015-2020 Kevin B. Hendricks, and Doug Massay
+# Copyright (c) 2014      Kevin B. Hendricks, John Schember, and Doug Massay
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without modification,
@@ -68,6 +69,15 @@ class InputContainer(object):
         # creates a new file not in manifest with desired ebook root relative href
         self._w.addotherfile(book_href, data)
 
+    # get path to hunspell dll / library
+    def get_hunspell_library_path(self):
+        return self._w.get_hunspell_path()
+
+    # get a list of the directories that contain Sigil's hunspell dictionaries
+    def get_dictionary_dirs(self):
+        return self._w.get_dictionary_dirs()
+
+    
     # New in Sigil 1.1
     # ----------------
 
