@@ -27,41 +27,13 @@
 
 import sys, os, codecs
 from collections import OrderedDict
-
-from urllib.parse import unquote
-from urllib.parse import urlsplit
-
-ASCII_CHARS   = set(chr(x) for x in range(128))
-URL_SAFE      = set('ABCDEFGHIJKLMNOPQRSTUVWXYZ'
-                    'abcdefghijklmnopqrstuvwxyz'
-                    '0123456789' '#' '_.-/~')
-IRI_UNSAFE = ASCII_CHARS - URL_SAFE
-
-def quoteurl(href):
-    if isinstance(href,bytes):
-        href = href.decode('utf-8')
-    (scheme, netloc, path, query, fragment) = urlsplit(href, scheme="", allow_fragments=True)
-    if scheme != "":
-        scheme += "://"
-        href = href[len(scheme):]
-    result = []
-    for char in href:
-        if char in IRI_UNSAFE:
-            char = "%%%02X" % ord(char)
-        result.append(char)
-    return scheme + ''.join(result)
-
-def unquoteurl(href):
-    if isinstance(href,bytes):
-        href = href.decode('utf-8')
-    href = unquote(href)
-    return href
+from hrefutils import urlencodepart, urldecodepart
 
 # encode to make xml safe
 def xmlencode(data):
     if data is None:
         return ''
-    newdata = data
+    newdata = xmldecode(data)
     newdata = newdata.replace('&', '&amp;')
     newdata = newdata.replace('<', '&lt;')
     newdata = newdata.replace('>', '&gt;')
@@ -102,8 +74,10 @@ class Opf_Parser(object):
         self.spine=[]
         self.guide=[]
         self.bindings=[]
+        self.ns_remap = False
         self._parseData()
 
+
     # OPF tag iterator
     def _opf_tag_iter(self):
         tcontent = last_tattr = None
@@ -116,6 +90,10 @@ class Opf_Parser(object):
                 tcontent = text.rstrip(" \r\n")
             else: # we have a tag
                 ttype, tname, tattr = self._parsetag(tag)
+                # remap opf namespace on tags if needed
+                if tname.startswith('opf:'):
+                    self.ns_remap = True
+                    tname = tname[4:]
                 if ttype == "begin":
                     tcontent = None
                     prefix.append(tname)
@@ -144,10 +122,17 @@ class Opf_Parser(object):
             if tname == "package":
                 ver = tattr.pop("version", "2.0")
                 uid = tattr.pop("unique-identifier","bookid")
+                if self.ns_remap:
+                    if "xmlns:opf" in tattr:
+                        tattr.pop("xmlns:opf")
+                        tattr["xmlns"] = "http://www.idpf.org/2007/opf"
                 self.package = (ver, uid, tattr)
                 continue
             # metadata
             if tname == "metadata":
+                if self.ns_remap:
+                    if not "xmlns:opf" in tattr:
+                        tattr["xmlns:opf"] = "http://www.idpf.org/2007/opf"
                 self.metadata_attr = tattr
                 continue
             if tname in ["meta", "link"] or tname.startswith("dc:") and "metadata" in prefix:
@@ -158,9 +143,13 @@ class Opf_Parser(object):
                 nid = "xid%03d" %  cnt
                 cnt += 1
                 id = tattr.pop("id",nid)
+                # must keep all hrefs in encoded) form
+                # if relative, then no fragments so decode and then encode for safety
                 href = tattr.pop("href","")
+                if href.find(':') == -1:
+                    href = urldecodepart(href)
+                    href = urlencodepart(href)
                 mtype = tattr.pop("media-type","")
-                href = unquoteurl(href)
                 self.manifest.append((id, href, mtype, tattr))
                 continue
             # spine
@@ -175,7 +164,8 @@ class Opf_Parser(object):
             if tname == "reference" and  "guide" in prefix:
                 type = tattr.pop("type","")
                 title = tattr.pop("title","")
-                href = unquoteurl(tattr.pop("href",""))
+                # must keep all hrefs in quoted (encoded) form
+                href = tattr.pop("href","")
                 self.guide.append((type, title, href))
                 continue
             # bindings
@@ -236,8 +226,8 @@ class Opf_Parser(object):
         while p < n and s[p:p+1] not in ('>', '/', ' ', '"', "'","\r","\n") : p += 1
         tname=s[b:p].lower()
         # remove redundant opf: namespace prefixes on opf tags
-        if tname.startswith("opf:"):
-            tname = tname[4:]
+        # if tname.startswith("opf:"):
+        #    tname = tname[4:]
         # more special cases
         if tname == '!doctype':
             tname = '!DOCTYPE'
@@ -368,8 +358,8 @@ class Opf_Parser(object):
     def convert_manifest_entries_to_xml(self):
         xmlres = []
         for (id, href, mtype, attr) in self.manifest:
-            url = quoteurl(href)
-            xmlres.append('    <item id="%s" href="%s" media-type="%s"' % (id, url, mtype))
+            # all hrefs should be kept in quoted (encoded) form 
+            xmlres.append('    <item id="%s" href="%s" media-type="%s"' % (id, href, mtype))
             for key in attr:
                 val= attr[key]
                 val= xmlencode(val)
@@ -404,8 +394,8 @@ class Opf_Parser(object):
     def convert_guide_entries_to_xml(self):
         xmlres=[]
         for (gtype, gtitle, ghref) in self.guide:
-            url = quoteurl(ghref)
-            xmlres.append('    <reference type="%s" title="%s" href="%s"/>\n' % (gtype, gtitle, url))
+            # all hrefs should already be in quoted (encoded) form
+            xmlres.append('    <reference type="%s" title="%s" href="%s"/>\n' % (gtype, gtitle, ghref))
         return "".join(xmlres)
                           
     def convert_binding_entries_to_xml(self):
