@@ -48,6 +48,8 @@ site_packages = [ ('lxml', 'd'),
                   ('regex', 'd'),
                   ('certifi', 'd'),
                   ('cssselect', 'd'),
+                  ('urllib3', 'd'),
+                  ('dulwich', 'd'),
                   ('encutils', 'd'),
                   ('css_parser', 'd'),
                   ('webencodings', 'd'), # needed by html5lib
@@ -215,7 +217,8 @@ def main():
     os.symlink(os.path.join('Versions', 'Current', 'Resources'), 'Resources')
 
     os.chdir(os.path.join(app_dir, 'Python.framework', 'Versions', pversion, 'lib'))
-    dylibname = 'libpython' + pversion + 'm.dylib'
+    # dylibname = 'libpython' + pversion + 'm.dylib'
+    dylibname = 'libpython' + pversion + '.dylib'
     os.symlink('../Python', dylibname)
 
     # Change any Python.framework rpaths in the Sigil executable to point to the new local Python.framework
