@@ -30,6 +30,9 @@ site_packages = [ ('lxml', 'd'),
                   ('PIL', 'd'), 
                   ('regex','d'),
                   ('cssselect', 'd'),
+                  ('urllib3', 'd'),
+                  ('certifi', 'd'),
+                  ('dulwich', 'd'),
                   ('encutils', 'd'),
                   ('css_parser', 'd'),
                   ('webencodings', 'd'), # needed by html5lib
